'use client';

import { useState, useEffect } from 'react';
import Image from 'next/image';
import {
  DropdownMenu,
  DropdownMenuContent,
  DropdownMenuLabel,
  DropdownMenuItem,
  DropdownMenuSeparator,
  DropdownMenuTrigger,
} from '@/components/ui/dropdown-menu';
import { createClient } from '@/utils/supabase/client';
import type { Session } from '@supabase/supabase-js';
import { Button } from '@/components/ui/button';
import type { User } from '@/lib/types';

const AvatarDropdown = () => {
  const supabase = createClient();
  const [session, setSession] = useState<Session | null>(null);
  const [loading, setLoading] = useState(true);
  const [user, setUser] = useState<User | null>(null);
  const [pfp, setPfp] = useState<string>('/images/default_user_pfp.png');

  useEffect(() => {
    // Function to fetch the session
    async function fetchSession() {
      try {
        const {
          data: { session },
        } = await supabase.auth.getSession();
        setSession(session);

        if (session?.user?.id) {
          const { data: userData, error } = await supabase
            .from('profiles')
            .select('*')
            .eq('id', session?.user.id)
            .single();
          if (error) {
            console.error('Error fetching user data:', error);
            return;
          }
          if (userData) {
            const user = userData as User;
            console.log(user);
            setUser(user);
          }
        }

      } catch (error) {
        console.error('Error fetching session:', error);
      } finally {
        setLoading(false);
      }
    }

    // Call the function
    fetchSession().catch((error) => {
      console.error('Error fetching session:', error);
    });

    // Set up auth state change listener
    const {
      data: { subscription },
    } = supabase.auth.onAuthStateChange((_event, session) => {
      setSession(session);
    });

    // Clean up the subscription when component unmounts
    return () => {
      subscription.unsubscribe();
    };
  }, [supabase]);

  useEffect(() => {
    if (user?.avatar_url) {
      console.log('Avatar Url:', user.avatar_url);
      if (user.avatar_url.startsWith('http')) {
        setPfp(user.avatar_url);
        return;
      }
      // Get public URL - this is synchronous
      const { data } = supabase
        .storage
        .from('avatars')
        .getPublicUrl(user.avatar_url);
      
      console.log('Avatar URL:', data.publicUrl);
      setPfp(data.publicUrl);
    }
  }, [user, supabase]);

  const getInitials = (fullName: string | undefined): string => {
    if (!fullName || fullName.trim() === '' || fullName === 'undefined') return 'NA';
    const nameParts = fullName.trim().split(' ');
    const firstInitial = nameParts[0]?.charAt(0).toUpperCase() ?? 'N';
    if (nameParts.length === 1) return 'NA';
    const lastIntitial = nameParts[nameParts.length -1]?.charAt(0).toUpperCase() ?? 'A';
    return firstInitial + lastIntitial;
  };

  // Handle sign out
  const handleSignOut = async () => {
    await supabase.auth.signOut();
  };

  // Show nothing while loading
  if (loading) {
    return <div className='animate-pulse h-8 w-8 rounded-full bg-gray-300' />;
  }

  // If no session, return empty div
  if (!session) {
    return <div />;
  }

  // If no session, return empty div
  if (!session) return <div />;
  return (
    <div className='m-auto mt-1'>
      <DropdownMenu>
        <DropdownMenuTrigger>
          <Image
            src={pfp}
            alt={getInitials(user?.full_name) ?? 'NA'}
            width={35}
            height={35}
            className='rounded-full border-2 border-white m-auto mr-1 md:mr-2
            max-w-[25px] md:max-w-[35px]'
          />
        </DropdownMenuTrigger>
        <DropdownMenuContent>
          <DropdownMenuLabel>{user?.full_name}</DropdownMenuLabel>
          <DropdownMenuSeparator />
          <DropdownMenuItem asChild>
            <Button onClick={handleSignOut} className='w-full text-left'>
              Sign Out
            </Button>
          </DropdownMenuItem>
        </DropdownMenuContent>
      </DropdownMenu>
    </div>
  );
};
export default AvatarDropdown;
